/*
 Copyright (C) JAEHYUK CHO
 All rights reserved.
 Code by JaeHyuk Cho <mailto:minzkn@minzkn.com>
*/

#if !defined(__def_mzapi_source_mzmixer_c__)
# define __def_mzapi_source_mzmixer_c__ "mzmixer.c"

#include <sys/types.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include "mzmixer.h"

#define __def_mzapi_inverse_level__ (0)

static struct ts_mzapi_mixer_device * (__mzapi_free_mixer_device__)(struct ts_mzapi_mixer_device *s_device);
static int (__mzapi_get_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int *s_mute, int *s_left, int *s_right);
static int (__mzapi_set_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int s_mute, int s_left, int s_right);
static struct ts_mzapi_mixer_device * (__mzapi_search_by_name_mixer__)(struct ts_mzapi_mixer *s_mixer, const char * s_name);
static struct ts_mzapi_mixer_device * (__mzapi_search_by_id_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id);
static int (__mzapi_set_volume_by_name_mixer__)(struct ts_mzapi_mixer *s_mixer, const char * s_name, int s_mute, int s_left, int s_right);
static int (__mzapi_set_volume_by_id_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int s_mute, int s_left, int s_right);
static int (__mzapi_get_volume_by_name_mixer__)(struct ts_mzapi_mixer *s_mixer, const char * s_name, int *s_mute, int *s_left, int *s_right);
static int (__mzapi_get_volume_by_id_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int *s_mute, int *s_left, int *s_right);

struct ts_mzapi_mixer * (mzapi_open_mixer)(const char * s_device_name);
struct ts_mzapi_mixer * (mzapi_close_mixer)(struct ts_mzapi_mixer *s_mixer);

static struct ts_mzapi_mixer_device * (__mzapi_free_mixer_device__)(struct ts_mzapi_mixer_device *s_device)
{
 struct ts_mzapi_mixer_device *s_prev;
 while(s_device != ((struct ts_mzapi_mixer_device *)0))
 {
  s_prev = s_device;
  s_device = s_device->next;
  if(s_prev->name != ((char *)0))free((void *)s_prev->name);
  free((void *)s_prev);
 }
 return((struct ts_mzapi_mixer_device *)0);
}

static int (__mzapi_get_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int *s_mute, int *s_left, int *s_right)
{
 int s_value = 0;
 if(ioctl(s_mixer->handle, MIXER_READ(s_id), &s_value, sizeof(s_value)) != 0)return(-1);
 if(s_mute != ((int *)0))*(s_mute) = (s_value & 0x8000) ? 1 : 0;
#if __def_mzapi_inverse_level__ == (0)
 if(s_left != ((int *)0))*(s_left) = ((s_value >> 0) & 0x7f) % (100 + 1);
 if(s_right != ((int *)0))*(s_right) = ((s_value >> 8) & 0x7f) % (100 + 1);
#else
 if(s_left != ((int *)0))*(s_left) = ((s_value >> 8) & 0x7f) % (100 + 1);
 if(s_right != ((int *)0))*(s_right) = ((s_value >> 0) & 0x7f) % (100 + 1);
#endif
 return(1);
}

static int (__mzapi_set_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int s_mute, int s_left, int s_right)
{
 int s_value = 0;
 if(ioctl(s_mixer->handle, MIXER_READ(s_id), &s_value, sizeof(s_value)) != 0)s_value = 0;
 if(s_mute == 1)s_value = (s_value & 0x7f7f) | 0x8000;
 else if(s_mute == 0)s_value = s_value & 0x7f7f;
 if(s_left >= 0)
 {
  if(s_left > 100)s_left = 100;
#if __def_mzapi_inverse_level__ == (0)
  s_value = (s_value & 0xff80) | s_left;
#else
  s_value = (s_value & 0x80ff) | (s_left << 8);
#endif
 }
 if(s_right >= 0)
 {
  if(s_right > 100)s_right = 100;
#if __def_mzapi_inverse_level__ == (0)
  s_value = (s_value & 0x80ff) | (s_right << 8);
#else
  s_value = (s_value & 0xff80) | s_right;
#endif
 }
 if(ioctl(s_mixer->handle, MIXER_WRITE(s_id), &s_value, sizeof(s_value)) == 0)return(1);
 return(-1);
}

static struct ts_mzapi_mixer_device * (__mzapi_search_by_name_mixer__)(struct ts_mzapi_mixer *s_mixer, const char * s_name)
{
 struct ts_mzapi_mixer_device *s_device;
 if((s_mixer == ((struct ts_mzapi_mixer *)0)) || (s_name == ((char *)0)))return((struct ts_mzapi_mixer_device *)0);
 s_device = s_mixer->device;
 while(s_device != ((struct ts_mzapi_mixer_device *)0))
 {
  if(s_device->name != ((char *)0)){ if(strcmp(s_device->name, s_name) == 0)break; } 
  s_device = s_device->next;
 }
 return(s_device);
}

static struct ts_mzapi_mixer_device * (__mzapi_search_by_id_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id)
{
 struct ts_mzapi_mixer_device *s_device;
 if(s_mixer == ((struct ts_mzapi_mixer *)0))return((struct ts_mzapi_mixer_device *)0);
 s_device = s_mixer->device;
 while(s_device != ((struct ts_mzapi_mixer_device *)0))
 {
  if(s_device->id == s_id)break;
  s_device = s_device->next;
 }
 return(s_device);
}

static int (__mzapi_set_volume_by_name_mixer__)(struct ts_mzapi_mixer *s_mixer, const char * s_name, int s_mute, int s_left, int s_right)
{
 struct ts_mzapi_mixer_device *s_device;
 if(s_mixer == ((struct ts_mzapi_mixer *)0))return(-1);
 s_device = __mzapi_search_by_name_mixer__(s_mixer, s_name);
 if(s_device != ((struct ts_mzapi_mixer_device *)0))return(__mzapi_set_mixer__(s_mixer, s_device->id, s_mute, s_left, s_right));
 return(-1);
}

static int (__mzapi_set_volume_by_id_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int s_mute, int s_left, int s_right)
{
 if(s_mixer == ((struct ts_mzapi_mixer *)0))return(-1);
 return(__mzapi_set_mixer__(s_mixer, s_id, s_mute, s_left, s_right));
}

static int (__mzapi_get_volume_by_name_mixer__)(struct ts_mzapi_mixer *s_mixer, const char * s_name, int *s_mute, int *s_left, int *s_right)
{
 struct ts_mzapi_mixer_device *s_device;
 if(s_mixer == ((struct ts_mzapi_mixer *)0))return(-1);
 s_device = __mzapi_search_by_name_mixer__(s_mixer, s_name);
 if(s_device != ((struct ts_mzapi_mixer_device *)0))return(__mzapi_get_mixer__(s_mixer, s_device->id, s_mute, s_left, s_right));
 return(-1);
}

static int (__mzapi_get_volume_by_id_mixer__)(struct ts_mzapi_mixer *s_mixer, int s_id, int *s_mute, int *s_left, int *s_right)
{
 if(s_mixer == ((struct ts_mzapi_mixer *)0))return(-1);
 return(__mzapi_get_mixer__(s_mixer, s_id, s_mute, s_left, s_right));
}

struct ts_mzapi_mixer * (mzapi_open_mixer)(const char * s_device_name)
{
 struct ts_mzapi_mixer *s_mixer = (struct ts_mzapi_mixer *)malloc((size_t)sizeof(struct ts_mzapi_mixer));
 if(s_mixer == ((struct ts_mzapi_mixer *)0))return((struct ts_mzapi_mixer *)0);
 s_mixer->device_name = (char *)0;
 s_mixer->handle = (-1);
 s_mixer->devices = 0;
 s_mixer->device = (struct ts_mzapi_mixer_device *)0;
 s_mixer->search_by_name = __mzapi_search_by_name_mixer__;
 s_mixer->search_by_id = __mzapi_search_by_id_mixer__;
 s_mixer->set_volume_by_name = __mzapi_set_volume_by_name_mixer__;
 s_mixer->set_volume_by_id = __mzapi_set_volume_by_id_mixer__;
 s_mixer->get_volume_by_name = __mzapi_get_volume_by_name_mixer__;
 s_mixer->get_volume_by_id = __mzapi_get_volume_by_id_mixer__;
 if(s_device_name != ((char *)0))s_mixer->device_name = strdup(s_device_name);
 else s_mixer->device_name = strdup("/dev/mixer");
 if(s_mixer->device_name != ((char *)0))
 {
  s_mixer->handle = open(s_mixer->device_name, O_RDONLY);
  if(s_mixer->handle != (-1))
  {
   static const char * s_mixer_name_table[] = SOUND_DEVICE_NAMES;
   int s_id, s_max_device, s_max_names;
   struct ts_mzapi_mixer_device *s_new_device, *s_last_device;
   s_last_device = s_mixer->device;
   s_max_names = sizeof(s_mixer_name_table) / sizeof(char *);
   s_max_device = SOUND_MIXER_NRDEVICES;
   if(s_max_device > s_max_names)s_max_device = s_max_names;
   for(s_id = 0;s_id < s_max_device;s_id++)
   { /* mixer device auto detection */
    if(__mzapi_get_mixer__(s_mixer, s_id, (int *)0, (int *)0, (int *)0) != 1)continue;
    s_new_device = (struct ts_mzapi_mixer_device *)malloc((size_t)sizeof(struct ts_mzapi_mixer_device));
    if(s_new_device == ((struct ts_mzapi_mixer_device *)0))continue;
    s_new_device->name = strdup(s_mixer_name_table[s_id]);
    if(s_new_device->name != ((char *)0))
    {
     s_new_device->id = s_id;
     if(s_last_device != ((struct ts_mzapi_mixer_device *)0))s_last_device->next = s_new_device;
     else s_mixer->device = s_new_device;
     s_last_device = s_new_device;
     s_mixer->devices++;
    }
    else s_new_device = __mzapi_free_mixer_device__(s_new_device); 
   }
  }
 }
 if(s_mixer->devices <= 0)s_mixer = mzapi_close_mixer(s_mixer);
 return(s_mixer);
}

struct ts_mzapi_mixer * (mzapi_close_mixer)(struct ts_mzapi_mixer *s_mixer)
{
 if(s_mixer == ((struct ts_mzapi_mixer *)0))return((struct ts_mzapi_mixer *)0);
 s_mixer->device = __mzapi_free_mixer_device__(s_mixer->device);
 if(s_mixer->handle != (-1))(void)close(s_mixer->handle);
 if(s_mixer->device_name != ((char *)0))free((void *)s_mixer->device_name);
 free((void *)s_mixer);
 return((struct ts_mzapi_mixer *)0);
}

#endif

/* vim: set expandtab: */
/* End of source */
